@@ -36,11 +36,12 @@
 #include <ipc/ipc_init.h>
 #include <kern/cpu_number.h>
 #include <kern/debug.h>
+#include <kern/gsync.h>
 #include <kern/machine.h>
 #include <kern/mach_factor.h>
 #include <kern/mach_clock.h>
-#include <kern/printf.h>
 #include <kern/processor.h>
+#include <kern/rdxtree.h>
 #include <kern/sched_prim.h>
 #include <kern/task.h>
 #include <kern/thread.h>
@@ -108,10 +109,10 @@ void setup_main(void)
 #endif	/* MACH_KDB */
 
 	panic_init();
-	printf_init();
 
 	sched_init();
 	vm_mem_bootstrap();
+	rdxtree_cache_init();
 	ipc_bootstrap();
 	vm_mem_init();
 	ipc_init();
@@ -136,7 +137,7 @@ void setup_main(void)
 	mapable_time_init();
 
 	machine_info.max_cpus = NCPUS;
-	machine_info.memory_size = phys_last_addr - phys_first_addr; /* XXX mem_size */
+	machine_info.memory_size = vm_page_mem_size(); /* XXX phys_addr_t -> vm_size_t */
 	machine_info.avail_cpus = 0;
 	machine_info.major_version = KERNEL_MAJOR_VERSION;
 	machine_info.minor_version = KERNEL_MINOR_VERSION;
@@ -158,6 +159,8 @@ void setup_main(void)
 	recompute_priorities(NULL);
 	compute_mach_factor();
 
+	gsync_setup ();
+
 	/*
 	 *	Create a kernel thread to start the other kernel
 	 *	threads.  Thread_resume (from kernel_thread) calls
